function [ ecoOutput ] = economic_analysis_biom( SimParam, BattParam, InvParam, InData, EcoParam, SimOut)
%ECONOMICANALYSIS The economic implications for every simulated solution
%   O(n^2) loops through every solution of the simulation to make an
%   economic solution space. Changing the SimPar will be sufficient to
%   calculate a different scope of solutions

[nBattEmployed, investmentCost, operationMaintenanceReplacementCost, ...
installBalanceOfSystemTotCost, bioSystemOperationCost, biomassPresentCost,...
bioSysNetPresentCost, battCostTot, pvCostTot, inverterCostTot, netPresentCost,...
levelizedCostOfEnergy, bioSysLevelizedCostOfEnergy]...
= deal(zeros(SimParam.nPvSteps, SimParam.nBattSteps));

capitalRecoveryFactor = (EcoParam.interestRate...
      * ((1 + EcoParam.interestRate)...
      ^ EcoParam.plantLifetimeYears))...
      / (((1 + EcoParam.interestRate)...
      ^ EcoParam.plantLifetimeYears) - 1);

for iPv = 1 : SimParam.nPvSteps 
    
    iPvKw = SimParam.pv_step_to_kw(iPv - 1);
    
    for jBatt = 1 : SimParam.nBattSteps
        
        jBattKwh = SimParam.batt_step_to_kwh(jBatt - 1);

        battCostTot(iPv, jBatt) = EcoParam.battCostKwh * jBattKwh...
                                    + EcoParam.battCostFixed;            

        loadPeakKw = max(InData.load);                                   
        
        %inverter is designed on the peak power value
        inverterCostTot(iPv, jBatt) = (loadPeakKw/InvParam.efficiency)...
                                        * EcoParam.inverterCostKw;                                       

        pvCostTot(iPv, jBatt) = EcoParam.pvCostKw...
                                  * iPvKw;
        
        % cost of Balance of System (BoS) and Installation
        installBalanceOfSystemTotCost(iPv, jBatt) = EcoParam.installBalanceOfSystemCost...
                                        * (battCostTot(iPv, jBatt)...
                                        + inverterCostTot(iPv, jBatt)...
                                        + pvCostTot(iPv, jBatt));              

        investmentCost(iPv,jBatt) = pvCostTot(iPv, jBatt) ...
                                  + battCostTot(iPv, jBatt) ...
                                  + inverterCostTot(iPv, jBatt) ...
                                  + installBalanceOfSystemTotCost(iPv, jBatt)...
                                  + EcoParam.bioSystemInvestmentCost;    
        
        operationMaintenanceCost = EcoParam.operationMaintenanceCostKw...
                                   * iPvKw;
                            
        % batteries should be replaced after this number of years
        battOperationalYears = 1/SimOut.sumPartialCyclesUsed(iPv,jBatt)...
                             * InData.nYears; 
        
        if battOperationalYears > BattParam.maxOperationalYears;
           battOperationalYears = BattParam.maxOperationalYears;
        end
        
        nBattEmployed(iPv,jBatt) = ceil(EcoParam.plantLifetimeYears ...
                                  / battOperationalYears);

        nextBatteryReplacementYear = battOperationalYears;
        for k = 1 : EcoParam.plantLifetimeYears
            if k > nextBatteryReplacementYear
                
                operationMaintenanceReplacementCost(iPv,jBatt)...
                                    = operationMaintenanceReplacementCost(iPv,jBatt)...
                                    + battCostTot(iPv, jBatt)...
                                    / ((1 + EcoParam.interestRate)...
                                    ^ battOperationalYears);      
                                    % computing present values of battery
                                    
                % batteries are replaced and new replacement year is set
                nextBatteryReplacementYear = nextBatteryReplacementYear ...
                                           + battOperationalYears;         
            end

            biomassPresentCost(iPv,jBatt) ...
                    = biomassPresentCost(iPv,jBatt)...
                    + (sum(SimOut.biomassGeneratorOutputKw(:,iPv, jBatt)...
                    * EcoParam.biomassCostKw))...
                    / (1 + EcoParam.interestRate)^k;
            
            bioSystemOperationCost(iPv, jBatt) ...
                                    = bioSystemOperationCost(iPv,jBatt) ...
                                    + (EcoParam.bioSystemOperationCostYear...
                                    / (1 + EcoParam.interestRate)^k);
                        
            
            % computing present values of Operations & Maintenance
            % and cost of biomass fuel production / consumage
            operationMaintenanceReplacementCost(iPv,jBatt) ...
                                    = operationMaintenanceReplacementCost(iPv,jBatt)...
                                    + (operationMaintenanceCost) ...
                                    / (1 + EcoParam.interestRate)^k;     
        end
        
        bioSystemOperationCost(iPv,jBatt) ...
                                  = bioSystemOperationCost(iPv,jBatt)...
                                  + biomassPresentCost(iPv,jBatt);
        
        % salvage due to battery life i.e. estimating
        % how much the batteries are worth after the
        % lifetime of the system
        operationMaintenanceReplacementCost(iPv,jBatt) ...
                                = operationMaintenanceReplacementCost(iPv,jBatt)...
                                - battCostTot(iPv, jBatt)...
                                * ( (nextBatteryReplacementYear ...
                                - EcoParam.plantLifetimeYears)...
                                / battOperationalYears ) ...
                                / (1 + EcoParam.interestRate)...
                                ^ (EcoParam.plantLifetimeYears);

                            
        % Adding inverter net present cost
        operationMaintenanceReplacementCost(iPv,jBatt) ...
                                = operationMaintenanceReplacementCost(iPv,jBatt)...
                                + inverterCostTot(iPv, jBatt)... 
                                / ((1 + EcoParam.interestRate)...
                                ^ (EcoParam.plantLifetimeYears / 2));
                            
        % adding the biomass system contribution to the sum
        operationMaintenanceReplacementCost(iPv,jBatt) ...
                            = operationMaintenanceReplacementCost(iPv,jBatt)...
                            + bioSystemOperationCost(iPv,jBatt);

        netPresentCost(iPv, jBatt) = investmentCost(iPv, jBatt)...
                                   + operationMaintenanceReplacementCost(iPv, jBatt);                               

        bioSysNetPresentCost(iPv,jBatt) = EcoParam.bioSystemInvestmentCost...
                                        + bioSystemOperationCost(iPv,jBatt);

        levelizedCostOfEnergy(iPv, jBatt) ...
                           = (netPresentCost(iPv, jBatt) ...
                           * capitalRecoveryFactor)...
                           ./ (sum(InData.load, 2)...
                           - SimOut.lossOfLoadTot(iPv, jBatt));

        bioSysLevelizedCostOfEnergy(iPv,jBatt) ...
                           = (bioSysNetPresentCost(iPv,jBatt)...
                           * capitalRecoveryFactor)...
                           / (sum(SimOut.biomassGeneratorOutputKw(:,iPv,jBatt))...
                           - sum(SimOut.biomassGeneratorOutputUnusedKw(:,iPv,jBatt)));
 
    end
end

% Levelized Cost of Energy i.e. cost per kWh of building and operating the 
% plant over an assumed life cycle. This is important as we want it to be competitive
% with the grid LCoE. See eqn. (7.6) in thesis Stefano Mandelli.

ecoOutput = EconomicAnalysisOutputs(investmentCost,... 
                                      netPresentCost,... 
                                      levelizedCostOfEnergy,...
                                      battCostTot,...
                                      inverterCostTot,...
                                      pvCostTot,...
                                      installBalanceOfSystemTotCost,...
                                      operationMaintenanceReplacementCost,...     
                                      capitalRecoveryFactor,...
                                      nBattEmployed,...
                                      bioSysNetPresentCost,...
                                      biomassPresentCost,...
                                      bioSysLevelizedCostOfEnergy);


end

